

from django.contrib import admin
from models import LogFile


class LogFileAdmin(admin.ModelAdmin):
    def has_add_permission(self, request):
        return False

admin.site.register(LogFile, LogFileAdmin)



from django.conf.urls import patterns, url
from views import *

urlpatterns = patterns('',
    url(r'^django_log_file_viewer/logfile/$', logfiles_list, {'template_name': 'logfiles_admin.html'}, name='log-files-list-admin'),
    url(r'^django_log_file_viewer/logfile/(?P<logfile_id>\d+)/$', logfile_view, {'template_name': 'logfile_admin.html'}, name='log-file-admin'),
    url(r'^django_log_file_viewer/logfile/(?P<logfile_id>\d+)/csv$', logfile_to_csv, name='log-file-to-csv-admin'),
)


import os, csv
from django.http import HttpResponse
from django.core.paginator import Paginator
from django.shortcuts import render_to_response
from django.http import HttpResponseBadRequest
from django.contrib.auth.decorators import login_required, user_passes_test
from settings import *
from models import LogFilesManager

__all__ = ['logfiles_list', 'logfile_view', 'logfile_to_csv']

@login_required
@user_passes_test(lambda u: u.is_superuser)
def logfiles_list(request, template_name='listfiles.html'):
    """Lists Log Files in settings directory"""
    manager = LogFilesManager()
    files_list = manager.list_logfiles(LOG_FILES_DIR)
    indexes = {}
    if files_list:
        count = 0
        for index in files_list:
            indexes[str(count)] = index
            count += 1
    return render_to_response(template_name, {'files_list': indexes,})

@login_required
@user_passes_test(lambda u: u.is_superuser)
def logfile_view(request, logfile_id, template_name='logfile.html'):
    """Returns a list of Log File content parsed by main regexp"""
    page = request.GET.get('page')
    if not page:
        page = 1
    else:
        try:
            page = int(page)
        except ValueError:
            page = 1
            pass

    manager = LogFilesManager()
    files_list = manager.list_logfiles(LOG_FILES_DIR)
    try:
        filename = files_list[int(logfile_id)]
    except Exception:
        return HttpResponseBadRequest()
    logfile = os.path.join(LOG_FILES_DIR, files_list[int(logfile_id)])
    log_file_object = manager.get_file(logfile)
    fake_log_file_lines = manager.get_file_lines_count(log_file_object)
    paginator = Paginator(fake_log_file_lines, LOG_FILES_PAGINATE_LINES)
    paginated_lines = paginator.page(page)
    first_line = paginated_lines.object_list[0]
    current_file_dict = manager.parse_log_file(logfile, first_line)
    paginated_lines.object_list = current_file_dict
    header_list = manager.compile_header_from_regexp()
    context = {
            'header_list': header_list,
            'file_name': filename,
            'file_id': logfile_id,
            'paginator': paginated_lines,
            'page': page,
    }
    return render_to_response(template_name, context)

@login_required
@user_passes_test(lambda u: u.is_superuser)
def logfile_to_csv(request, logfile_id):
    """Exports selected log file to PDF"""
    manager = LogFilesManager()
    files_list = manager.list_logfiles(LOG_FILES_DIR)
    try:
        filename = files_list[int(logfile_id)]
    except Exception:
        return HttpResponseBadRequest()
    logfile = os.path.join(LOG_FILES_DIR, files_list[int(logfile_id)])
    file_dict = manager.parse_log_file(logfile, 0, full=True)
    # Constructing CSV file
    header_list = manager.compile_header_from_regexp()
    response = HttpResponse(mimetype='text/csv')
    response['Content-Disposition'] = 'attachment; filename=%s.csv' % filename
    writer = csv.writer(response)
    if header_list:
        writer.writerow(header_list)
    if file_dict:
        for item in file_dict:
            row_list = []
            for atom in item:
                for cell in atom:
                    row_list += [unicode(cell).encode('utf8'),]
                writer.writerow(row_list)
    return response





from django.conf.urls import patterns, url
from views import *

urlpatterns = patterns('',
    url(r'^logfiles/$', logfiles_list, name='logfiles_list'),
    url(r'^logfiles/(?P<logfile_id>\d+)$', logfile_view, name='logfile_view'),
)



# Default settings for the app override in your settings.py
from django.conf import settings
import os

APP_PATH = os.path.abspath(os.path.split(__file__)[0])
LOG_FILES_DIR = getattr(
    settings,
    'LOG_FILES_DIR',
    os.path.join(APP_PATH, 'testdata', 'log')
)
LOG_FILES_RE = getattr(
    settings,
    'LOG_FILES_RE',
    '(?P<date>\d{4}-\d{2}-\d{2} \d{2}:\d{2}:\d{2},\d{3})\s\[(?P<type>[A-Z]+)\]\s(?P<message>.+)'
)

LOG_FILES_PAGINATE_LINES = getattr(settings, 'LOG_FILES_PAGINATE_LINES', 20)

PAGINATOR_PAGE_SEPARATOR = getattr(settings, 'PAGINATOR_PAGE_SEPARATOR', '...')

__all__ = ['LOG_FILES_DIR', 'LOG_FILES_RE', 'LOG_FILES_PAGINATE_LINES',]


#!/usr/bin/env python

from distutils.core import setup

setup(name='django-log-file-viewer',
    version='0.8',
    description='Django admin expansion to read/parse file based Django Logging output.',
    author='Iurii Garmash',
    author_email='garmon1@gmail.com',
    url='http://garmoncheg.blogspot.com/2012/09/django-log-files-viewer-documents.html',
    packages=['django-log-file-viewer'],
    package_dir={'django-log-file_viewer': 'src/django-log-file-viewer'},
    package_data={'django-log-file-viewer': [
        'templates/*.html',
        'templates/*.html',
        'testdata/log/test/.gitignore', # To keep this directory (required for farther tests)
        'testdata/log/testlog.log',
        'README.rst',
        'README',
        ]},
    classifiers=[
        'Development Status :: 4 - Beta',
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'Intended Audience :: System Administrators',
        'License :: OSI Approved :: GNU General Public License v3 or later (GPLv3+)',
        'Natural Language :: English',
        'Operating System :: MacOS :: MacOS X',
        'Operating System :: Microsoft :: Windows',
        'Operating System :: POSIX',
        'Programming Language :: Python',
        'Topic :: Internet :: Log Analysis',
        'Topic :: System :: Logging',
        'Topic :: System :: Systems Administration',
        'Topic :: Text Processing',
        ],
)


"""
Logging reader
"""
import os
import re
import linecache

from django.db import models

from settings import *

class LogFilesManager(object):

    def list_logfiles(self, path):
        """Returns list of files in provided path"""
        file_list = []
        # List only files
        for root,directory,files in os.walk(path):
            for f in files:
                # List only readable files
                try:
                    fi = open(os.path.join(path, f))
                    fi.close()
                    file_list.append(f)
                except Exception:
                    pass
        return file_list

    def get_file(self, file_full_path):
        fobj = open(file_full_path, 'rb')
        return fobj

    def get_file_lines_count(self, file_obj):
        """Creates fake log file list (without real lines, but with proper length)"""
        fake_log_file = []
        log_file_fake_lines = file_obj.xreadlines()
        count = 0
        for line in log_file_fake_lines:
            fake_log_file.append(count)
            count += 1
        if fake_log_file:
            fake_log_file.append(count)
        return fake_log_file

    def compile_re_index(self, regexp=None):
        """Creating Regexp prog to match entries"""
        if not regexp:
            regexp = LOG_FILES_RE
        prog = re.compile(regexp)
        return prog

    def parse_log_file(self, logfile, from_line=0, to_line=LOG_FILES_PAGINATE_LINES, full=False):
        """Returns parsed read file

        in form of entry names header (taken from Rgex group names)
        and lines tuples list"""
        file_dict = []
        prog = self.compile_re_index()
        # Reading amount of lines
        line_num = from_line
        if full:
            file_obj = self.get_file(logfile)
            to_line = self.get_file_lines_count(file_obj).__len__()
        for count in range(to_line):
            try:
                line = linecache.getline(logfile, line_num)
                matches_set = prog.findall(str(line))
                file_dict.append(matches_set)
                line_num += 1
            except IndexError:
                # log file is shorter then LOG_FILES_PAGINATE_LINES or
                # amount of lines smaller then from_line left
                pass
        return file_dict

    def compile_header_from_regexp(self, regexp=None):
        """Making logfile indexes header"""
        prog = self.compile_re_index()
        header_length = prog.groups
        header_list = []
        if prog.groupindex:
            for number in range(header_length):
                header_list.append(number)
            for group_name, index in prog.groupindex.iteritems():
                header_list[int(index) - 1] = group_name
        return header_list

class string_with_title(str):
    def __new__(cls, value, title):
        instance = str.__new__(cls, value)
        instance._title = title
        return instance

    def title(self):
        return self._title

    __copy__ = lambda self: self
    __deepcopy__ = lambda self, memodict: self

class LogFile(models.Model):
    """Hack object to be added to Django admin"""
    class Meta:
        app_label = string_with_title("django_log_file_viewer", "Django Log Files")
    verbose_name = 'Django Log File'
    verbose_name_plural = 'Django Log Files'



"""
This file demonstrates writing tests using the unittest module. These will pass
when you run "manage.py test".

Replace this with more appropriate tests for your application.
"""
import os
from django.test import TestCase
from models import LogFilesManager
from settings import *


class SimpleTest(TestCase):
    def test_basic_addition(self):
        """
        Tests that 1 + 1 always equals 2.
        """
        self.assertEqual(1 + 1, 2)

# class LogFilesManagerTests(TestCase):
#     def test_read_certain_log_file_lines(self):
#         manager = LogFilesManager()
#         files_list = manager.list_logfiles(LOG_FILES_DIR)
#         logfile = manager.get_file(os.path.join(LOG_FILES_DIR, files_list[int(0)]))
#         file_len, header_list, file_dict = manager.parse_log_file(logfile, from_line=100)
#         print file_dict
